like_coercion

Function like_coercion 

pub fn like_coercion(
    lhs_type: &DataType,
    rhs_type: &DataType,
) -> Option<DataType>
Coercion rules for like operations. This is a union of string coercion rules, dictionary coercion rules, and REE coercion rules Note: list_coercion is intentionally NOT included here because LIKE is a string pattern matching operation and is not supported for nested types (List, Struct, etc.)