Function datafusion_expr::binary_rule::comparison_coercion
source · [−]Expand description
Get the coerced data type for comparison operations such as eq, not eq, lt, lteq, gt, and gteq.
Get the coerced data type for comparison operations such as eq, not eq, lt, lteq, gt, and gteq.