Module expr

Module expr 

Logical Expressions: Expr

Structs

AggregateFunction
Aggregate function
AggregateFunctionParams
Alias
Alias expression
Between
BETWEEN expression
BinaryExpr
Binary expression for Expr::BinaryExpr
Case
CASE expression
Cast
Cast expression
ExceptSelectItemsql
Bigquery EXCEPT information, with at least one column.
Exists
EXISTS expression
ExprListDisplay
Formats a list of &Expr with a custom separator using SQL display format
FieldMetadata
Literal metadata
HigherOrderFunction
Invoke a HigherOrderUDF with a set of arguments
IlikeSelectItemsql
Snowflake ILIKE information.
InList
InList expression
InSubquery
IN subquery
Lambda
A Lambda expression with a set of parameters names and a body
LambdaVariable
A named reference to a lambda parameter which includes it’s own FieldRef, which is used to implement ExprSchemable, for example. It is an option only to make easier for expr_api users to construct lambda variables, but any expression tree or LogicalPlan containing unresolved variables must be resolved before usage with either Expr::resolve_lambda_variables or LogicalPlan::resolve_lambda_variables. The default SQL planner produces already resolved variables and no further resolving is required.
Like
LIKE expression
Placeholder
Placeholder, representing bind parameter values such as $1 or $name.
PlannedReplaceSelectItem
The planned expressions for REPLACE
ReplaceSelectElementsql
Syntax
ScalarFunction
Invoke a ScalarUDF with a set of arguments
SetComparison
Set comparison subquery (e.g. = ANY, > ALL)
Sort
SORT expression
TryCast
TryCast Expression
Unnest
UNNEST expression.
WildcardOptions
Additional options for wildcards, e.g. Snowflake EXCLUDE/RENAME and Bigquery EXCEPT.
WindowFunction
Window function
WindowFunctionParams

Enums

ExcludeSelectItemsql
Snowflake EXCLUDE information.
Expr
Represents logical expressions such as A + 1, or CAST(c1 AS int).
GetFieldAccess
Access a sub field of a nested type, such as Field or List
GroupingSet
Grouping sets
NullTreatment
RenameSelectItemsql
Snowflake RENAME information.
SetQuantifier
Whether the set comparison uses ANY/SOME or ALL
WindowFunctionDefinition
A function used as a SQL window function

Constants

OUTER_REFERENCE_COLUMN_PREFIX
UNNEST_COLUMN_PREFIX

Functions

display_comma_separated
intersect_metadata_for_union
Intersects multiple metadata instances for UNION operations.
physical_name
The name of the column (field) that this Expr will produce in the physical plan. The difference from Expr::schema_name is that top-level columns are unqualified.
schema_name_from_exprs
Get schema_name for Vector of expressions
schema_name_from_sorts

Type Aliases

SchemaFieldMetadata
The metadata used in Field::metadata.