pub struct Unnest {
pub expr: Box<Expr>,
pub outer: bool,
}Expand description
UNNEST expression.
When outer is true, the unnest should preserve NULL and empty input
lists by emitting a single NULL output row for each. When false (the
historical default), the behavior is identical to the plain UNNEST(col)
SQL form: NULL and empty input lists are dropped from the output.
Fields
expr: Box<Expr>outer: boolOuter-unnest behavior: also expand empty input lists into a single
NULL output row (in addition to preserving NULL input rows).
Implementations