pub enum DdlStatement {
CreateExternalTable(Box<CreateExternalTable>),
CreateMemoryTable(CreateMemoryTable),
CreateView(CreateView),
CreateCatalogSchema(CreateCatalogSchema),
CreateCatalog(CreateCatalog),
CreateIndex(CreateIndex),
DropTable(DropTable),
DropView(DropView),
DropCatalogSchema(DropCatalogSchema),
CreateFunction(Box<CreateFunction>),
DropFunction(DropFunction),
}Expand description
Various types of DDL (CREATE / DROP) catalog manipulation
Variants
CreateExternalTable(Box<CreateExternalTable>)
Creates an external table. Boxed to keep LogicalPlan enum size down
— CreateExternalTable is ~312 bytes, dwarfing every other variant
in the plan tree and forcing the whole enum to that width.
CreateMemoryTable(CreateMemoryTable)
Creates an in memory table.
CreateView(CreateView)
Creates a new view.
CreateCatalogSchema(CreateCatalogSchema)
Creates a new catalog schema.
CreateCatalog(CreateCatalog)
Creates a new catalog (aka “Database”).
CreateIndex(CreateIndex)
Creates a new index.
DropTable(DropTable)
Drops a table.
DropView(DropView)
Drops a view.
DropCatalogSchema(DropCatalogSchema)
Drops a catalog schema
CreateFunction(Box<CreateFunction>)
Create function statement. Boxed for the same reason as
Self::CreateExternalTable (~288 bytes).
DropFunction(DropFunction)
Drop function statement
Implementations
Sourceimpl DdlStatement
impl DdlStatement
Sourcepub fn schema(&self) -> &DFSchemaRef
pub fn schema(&self) -> &DFSchemaRef
Get a reference to the logical plan’s schema
Sourcepub fn name(&self) -> &str
pub fn name(&self) -> &str
Return a descriptive string describing the type of this
DdlStatement
Sourcepub fn inputs(&self) -> Vec<&LogicalPlan>
pub fn inputs(&self) -> Vec<&LogicalPlan>
Return all inputs for this plan
Sourcepub fn display(&self) -> impl Display + '_
pub fn display(&self) -> impl Display + '_
Return a formatable structure with the a human readable
description of this LogicalPlan node per node, not including
children.
See crate::LogicalPlan::display for an example