ExplainFormat

Enum ExplainFormat 

pub enum ExplainFormat {
    Indent,
    Tree,
    PostgresJSON,
    Graphviz,
}
Output formats for controlling for Explain plans

Variants



Indent

Indent mode

Example:

> explain format indent select x from values (1) t(x);
+---------------+-----------------------------------------------------+
| plan_type     | plan                                                |
+---------------+-----------------------------------------------------+
| logical_plan  | SubqueryAlias: t                                    |
|               |   Projection: column1 AS x                          |
|               |     Values: (Int64(1))                              |
| physical_plan | ProjectionExec: expr=[column1@0 as x]               |
|               |   DataSourceExec: partitions=1, partition_sizes=[1] |
|               |                                                     |
+---------------+-----------------------------------------------------+


Tree

Tree mode

Example:

> explain format tree select x from values (1) t(x);
+---------------+-------------------------------+
| plan_type     | plan                          |
+---------------+-------------------------------+
| physical_plan | ┌───────────────────────────┐ |
|               | │       ProjectionExec      │ |
|               | │    --------------------   │ |
|               | │        x: column1@0       │ |
|               | └─────────────┬─────────────┘ |
|               | ┌─────────────┴─────────────┐ |
|               | │       DataSourceExec      │ |
|               | │    --------------------   │ |
|               | │         bytes: 128        │ |
|               | │       format: memory      │ |
|               | │          rows: 1          │ |
|               | └───────────────────────────┘ |
|               |                               |
+---------------+-------------------------------+


PostgresJSON

Postgres Json mode

A displayable structure that produces plan in postgresql JSON format.

Users can use this format to visualize the plan in existing plan visualization tools, for example dalibo

Example:

> explain format pgjson select x from values (1) t(x);
+--------------+--------------------------------------+
| plan_type    | plan                                 |
+--------------+--------------------------------------+
| logical_plan | [                                    |
|              |   {                                  |
|              |     "Plan": {                        |
|              |       "Alias": "t",                  |
|              |       "Node Type": "Subquery",       |
|              |       "Output": [                    |
|              |         "x"                          |
|              |       ],                             |
|              |       "Plans": [                     |
|              |         {                            |
|              |           "Expressions": [           |
|              |             "column1 AS x"           |
|              |           ],                         |
|              |           "Node Type": "Projection", |
|              |           "Output": [                |
|              |             "x"                      |
|              |           ],                         |
|              |           "Plans": [                 |
|              |             {                        |
|              |               "Node Type": "Values", |
|              |               "Output": [            |
|              |                 "column1"            |
|              |               ],                     |
|              |               "Plans": [],           |
|              |               "Values": "(Int64(1))" |
|              |             }                        |
|              |           ]                          |
|              |         }                            |
|              |       ]                              |
|              |     }                                |
|              |   }                                  |
|              | ]                                    |
+--------------+--------------------------------------+


Graphviz

Graphviz mode

Example:

> explain format graphviz select x from values (1) t(x);
+--------------+------------------------------------------------------------------------+
| plan_type    | plan                                                                   |
+--------------+------------------------------------------------------------------------+
| logical_plan |                                                                        |
|              | // Begin DataFusion GraphViz Plan,                                     |
|              | // display it online here: https://dreampuf.github.io/GraphvizOnline   |
|              |                                                                        |
|              | digraph {                                                              |
|              |   subgraph cluster_1                                                   |
|              |   {                                                                    |
|              |     graph[label="LogicalPlan"]                                         |
|              |     2[shape=box label="SubqueryAlias: t"]                              |
|              |     3[shape=box label="Projection: column1 AS x"]                      |
|              |     2 -> 3 [arrowhead=none, arrowtail=normal, dir=back]                |
|              |     4[shape=box label="Values: (Int64(1))"]                            |
|              |     3 -> 4 [arrowhead=none, arrowtail=normal, dir=back]                |
|              |   }                                                                    |
|              |   subgraph cluster_5                                                   |
|              |   {                                                                    |
|              |     graph[label="Detailed LogicalPlan"]                                |
|              |     6[shape=box label="SubqueryAlias: t\nSchema: [x:Int64;N]"]         |
|              |     7[shape=box label="Projection: column1 AS x\nSchema: [x:Int64;N]"] |
|              |     6 -> 7 [arrowhead=none, arrowtail=normal, dir=back]                |
|              |     8[shape=box label="Values: (Int64(1))\nSchema: [column1:Int64;N]"] |
|              |     7 -> 8 [arrowhead=none, arrowtail=normal, dir=back]                |
|              |   }                                                                    |
|              | }                                                                      |
|              | // End DataFusion GraphViz Plan                                        |
|              |                                                                        |
+--------------+------------------------------------------------------------------------+