pub enum Partitioning {
RoundRobinBatch(usize),
Hash(Vec<Expr>, usize),
Range(RangePartitioning),
DistributeBy(Vec<Expr>),
}Expand description
Logical partitioning schemes.
A scheme can describe either requested repartitioning in
LogicalPlan::Repartition or a partitioning property declared by a source.
Some schemes are only valid as metadata until planner support is added.
For physical execution partitioning, see
datafusion_physical_expr::Partitioning.
Variants
RoundRobinBatch(usize)
Allocate batches using a round-robin algorithm and the specified number of partitions
Hash(Vec<Expr>, usize)
Allocate rows based on a hash of one of more expressions and the specified number of partitions.
Range(RangePartitioning)
Partition rows by ranges.
See RangePartitioning for the logical contract.
DistributeBy(Vec<Expr>)
The DISTRIBUTE BY clause is used to repartition the data based on the input expressions
Implementations
Sourceimpl Partitioning
impl Partitioning
Sourcepub fn partition_count(&self) -> Option<usize>
pub fn partition_count(&self) -> Option<usize>
Return the number of partitions, if known.