CreateExternalTable

Struct CreateExternalTable 

pub struct CreateExternalTable {
Show 14 fields pub schema: DFSchemaRef, pub name: TableReference, pub locations: Vec<String>, pub file_type: String, pub table_partition_cols: Vec<String>, pub if_not_exists: bool, pub or_replace: bool, pub temporary: bool, pub definition: Option<String>, pub order_exprs: Vec<Vec<Sort>>, pub unbounded: bool, pub options: HashMap<String, String>, pub constraints: Constraints, pub column_defaults: HashMap<String, Expr>,
}
Creates an external table.

Fields

schema: DFSchemaRef

The table schema

name: TableReference

The table name

locations: Vec<String>

The physical locations of the table files.

More than one location may be supplied (for example CREATE EXTERNAL TABLE ... LOCATION ('a.parquet', 'b.parquet')), in which case the files are read together as a single table.

file_type: String

The file type of physical file

table_partition_cols: Vec<String>

Partition Columns

if_not_exists: bool

Option to not error if table already exists

or_replace: bool

Option to replace table content if table already exists

temporary: bool

Whether the table is a temporary table

definition: Option<String>

SQL used to create the table, if available

order_exprs: Vec<Vec<Sort>>

Order expressions supplied by user

unbounded: bool

Whether the table is an infinite streams

options: HashMap<String, String>

Table(provider) specific options

constraints: Constraints

The list of constraints in the schema, such as primary key, unique, etc.

column_defaults: HashMap<String, Expr>

Default values for columns

Implementations

Source

impl CreateExternalTable

pub fn builder( name: impl Into<TableReference>, location: impl Into<String>, file_type: impl Into<String>, schema: DFSchemaRef, ) -> CreateExternalTableBuilder

Creates a builder for CreateExternalTable with required fields.

Arguments
  • name - The table name
  • location - The physical location of the table files
  • file_type - The file type (e.g., “parquet”, “csv”, “json”)
  • schema - The table schema
Example
let table = CreateExternalTable::builder(
    TableReference::bare("my_table"),
    "/path/to/data",
    "parquet",
    Arc::new(DFSchema::empty())
).build();