pub struct TableScan {
pub table_name: TableReference,
pub source: Arc<dyn TableSource>,
pub projection: Option<Vec<usize>>,
pub projected_schema: DFSchemaRef,
pub filters: Vec<Expr>,
pub fetch: Option<usize>,
pub statistics_requests: BTreeSet<StatisticsRequest>,
}Expand description
Produces rows from a table provider by reference or from the context
Fields
table_name: TableReferenceThe name of the table
source: Arc<dyn TableSource>The source of the table
projection: Option<Vec<usize>>Optional column indices to use as a projection
projected_schema: DFSchemaRefThe schema description of the output
filters: Vec<Expr>Optional expressions to be used as filters by the table provider
fetch: Option<usize>Optional number of rows to read
statistics_requests: BTreeSet<StatisticsRequest>Statistics the planner would like the provider to answer for this scan, typically attached by a custom optimizer rule from the surrounding plan (e.g. Min/Max for sort keys).
A BTreeSet, not a Vec to keep the resulting plan deterministic.
Implementations
Sourceimpl TableScan
impl TableScan
Sourcepub fn try_new(
table_name: impl Into<TableReference>,
table_source: Arc<dyn TableSource>,
projection: Option<Vec<usize>>,
filters: Vec<Expr>,
fetch: Option<usize>,
) -> Result<Self>
👎Deprecated since 54.0.0: use TableScanBuilder instead
pub fn try_new( table_name: impl Into<TableReference>, table_source: Arc<dyn TableSource>, projection: Option<Vec<usize>>, filters: Vec<Expr>, fetch: Option<usize>, ) -> Result<Self>
use TableScanBuilder instead
Initialize TableScan with appropriate schema from the given arguments.