Module registry

Module registry 

FunctionRegistry trait

Structs

ExtensionTypeRegistration
The registration of an extension type. Implementations of this trait are responsible for creating instances of DFExtensionType that represent the entire semantics of an extension type.
MemoryExtensionTypeRegistry
An ExtensionTypeRegistry that uses in memory HashMaps.
MemoryFunctionRegistry
A FunctionRegistry that uses in memory HashMaps

Traits

ExtensionTypeRegistry
Manages ExtensionTypeRegistrations, which allow users to register custom behavior for extension types.
FunctionRegistry
A registry knows how to build logical expressions out of user-defined function’ names
SerializerRegistry
Serializer and deserializer registry for extensions like UserDefinedLogicalNode.

Type Aliases

ExtensionTypeFactory
A factory that creates instances of extension types from a storage DataType and the metadata.
ExtensionTypeRegistrationRef
A cheaply cloneable pointer to an ExtensionTypeRegistration.
ExtensionTypeRegistryRef
A cheaply cloneable pointer to an ExtensionTypeRegistry.