ExtensionTypeRegistration

Struct ExtensionTypeRegistration 

pub struct ExtensionTypeRegistration { /* private fields */ }
The registration of an extension type. Implementations of this trait are responsible for creating instances of DFExtensionType that represent the entire semantics of an extension type.

Why do we need a Registration?

A good question is why this trait is even necessary. Why not directly register the DFExtensionType in a registry?

While this works for extension types requiring no additional metadata (e.g., arrow.uuid), it does not work for more complex extension types with metadata. For example, consider an extension type custom.shortened(n) that aims to short the pretty-printing string to n characters. Here, n is a parameter of the extension type and should be a field in the struct that implements the DFExtensionType. The job of the registration is to read the metadata from the field and create the corresponding DFExtensionType instance with the correct n set.

Implementations

Source

impl ExtensionTypeRegistration

pub fn new_arc( name: impl Into<String>, factory: impl Fn(&DataType, Option<&str>) -> Result<DFExtensionTypeRef> + Send + Sync + 'static, ) -> ExtensionTypeRegistrationRef

Creates a new registration for an extension type. The factory is required to validate that the storage DataType is compatible with the extension type.

Source

impl ExtensionTypeRegistration

pub fn type_name(&self) -> &str

The name of the extension type.

This name will be used to find the correct ExtensionTypeRegistration when an extension type is encountered.

pub fn create_df_extension_type( &self, storage_type: &DataType, metadata: Option<&str>, ) -> Result<DFExtensionTypeRef>

Creates an extension type instance from the optional metadata. The name of the extension type is not a parameter as it’s already defined by the registration itself.