MemoryExtensionTypeRegistry

Struct MemoryExtensionTypeRegistry 

pub struct MemoryExtensionTypeRegistry { /* private fields */ }
An ExtensionTypeRegistry that uses in memory HashMaps.

Implementations

Source

impl MemoryExtensionTypeRegistry

pub fn new_empty() -> Self

Creates an empty MemoryExtensionTypeRegistry.

pub fn new_with_canonical_extension_types() -> Self

Pre-registers the canonical extension types in the extension type registry.

pub fn new_with_types( types: impl IntoIterator<Item = ExtensionTypeRegistrationRef>, ) -> Result<Self>

Creates a new MemoryExtensionTypeRegistry with the provided types.

Errors

Returns an error if one of the types is a native type.

pub fn all_extension_types(&self) -> Vec<ExtensionTypeRegistrationRef>

Returns a list of all registered types.