combine_bernoullis

Function combine_bernoullis 

pub fn combine_bernoullis(
    op: &Operator,
    left: &BernoulliDistribution,
    right: &BernoulliDistribution,
) -> Result<BernoulliDistribution, DataFusionError>
👎Deprecated since 54.0.0:

Part of the unused Statistics V2 framework; see https://github.com/apache/datafusion/pull/22071

This function takes a logical operator and two Bernoulli distributions, and it returns a new Bernoulli distribution that represents the result of the operation. Currently, only AND and OR operations are supported.