pub fn combine_gaussians(
op: &Operator,
left: &GaussianDistribution,
right: &GaussianDistribution,
) -> Result<Option<GaussianDistribution>, DataFusionError>👎Deprecated since 54.0.0:
Part of the unused Statistics V2 framework; see https://github.com/apache/datafusion/pull/22071
Applies the given operation to the given Gaussian distributions. Currently,
this function handles only addition and subtraction operations. If the
result is not a Gaussian random variable, it returns None. For details,
see:
https://en.wikipedia.org/wiki/Sum_of_normally_distributed_random_variables