compute_median

Function compute_median 

pub fn compute_median(
    op: &Operator,
    left: &Distribution,
    right: &Distribution,
) -> Result<ScalarValue, DataFusionError>
👎Deprecated since 54.0.0:

Part of the unused Statistics V2 framework; see https://github.com/apache/datafusion/pull/22071

Computes the median value for the result of the given binary operation on two unknown quantities represented by its Distribution objects. Currently, the median is calculable only for addition and subtraction operations on: