pub fn create_bernoulli_from_comparison(
op: &Operator,
left: &Distribution,
right: &Distribution,
) -> Result<Distribution, DataFusionError>👎Deprecated since 54.0.0:
Part of the unused Statistics V2 framework; see https://github.com/apache/datafusion/pull/22071
Creates a new Bernoulli distribution by computing the resulting probability.
Expects op to be a comparison operator, with left and right having
numeric distributions. The resulting distribution has the Float64 data
type.