BernoulliDistribution

Struct BernoulliDistribution 

pub struct BernoulliDistribution { /* private fields */ }
๐Ÿ‘ŽDeprecated since 54.0.0:

Part of the unused Statistics V2 framework; see https://github.com/apache/datafusion/pull/22071

Bernoulli distribution with success probability p. If p has a null value, the success probability is unknown. For a more in-depth discussion, see:

https://en.wikipedia.org/wiki/Bernoulli_distribution

Implementations

Source

impl BernoulliDistribution

pub fn data_type(&self) -> DataType

pub fn p_value(&self) -> &ScalarValue

pub fn mean(&self) -> &ScalarValue

pub fn median(&self) -> Result<ScalarValue, DataFusionError>

Computes the median value of this distribution. In case of an unknown success probability, the function returns a NULL ScalarValue.

pub fn variance(&self) -> Result<ScalarValue, DataFusionError>

Computes the variance value of this distribution. In case of an unknown success probability, the function returns a NULL ScalarValue.

pub fn range(&self) -> Interval