HigherOrderSignature

Struct HigherOrderSignature 

pub struct HigherOrderSignature {
    pub type_signature: HigherOrderTypeSignature,
    pub volatility: Volatility,
    pub lambda_parameters_max_iterations: usize,
}
Provides information necessary for calling a higher order function.

Fields

type_signature: HigherOrderTypeSignature

The data types that the function accepts. See HigherOrderTypeSignature for more information.

volatility: Volatility

The volatility of the function. See Volatility for more information.

lambda_parameters_max_iterations: usize

The max number of times to call HigherOrderUDFImpl::lambda_parameters before raising an error. Used to guard against implementations that causes an infinite loop by endlessly returning LambdaParametersProgress::Partial. Defaults to 256

Implementations

Source

impl HigherOrderSignature

pub fn new( type_signature: HigherOrderTypeSignature, volatility: Volatility, ) -> Self

Creates a new HigherOrderSignature from a given type signature and volatility.

pub fn user_defined(volatility: Volatility) -> Self

User-defined coercion rules for the function.

pub fn variadic_any(volatility: Volatility) -> Self

An arbitrary number of lambdas or arguments of any type.

pub fn any(arg_count: usize, volatility: Volatility) -> Self

A specified number of arguments of any type

pub fn exact(args: Vec<ValueOrLambda<(), ()>>, volatility: Volatility) -> Self

Exactly the specified arguments in the given order, with arbitrary types. DataFusion will call HigherOrderUDFImpl::coerce_value_types to prepare the value argument types.

Example

A function that takes one value argument followed by one lambda:

let sig = HigherOrderSignature::exact(
    vec![ValueOrLambda::Value(()), ValueOrLambda::Lambda(())],
    Volatility::Immutable,
);