HigherOrderUDF

Struct HigherOrderUDF 

pub struct HigherOrderUDF { /* private fields */ }
Logical representation of a Higher Order User Defined Function.

A higher order function takes one or more lambda arguments in addition to regular value arguments. This struct contains the information DataFusion needs to plan and invoke functions you supply such as name, type signature, return type, and actual implementation.

Implementations

Source

impl HigherOrderUDF

pub fn new_from_impl<F>(fun: F) -> HigherOrderUDF
where F: HigherOrderUDFImpl + 'static,

Create a new HigherOrderUDF from a HigherOrderUDFImpl trait object.

Note this is the same as using the From impl (HigherOrderUDF::from).

pub fn new_from_shared_impl(fun: Arc<dyn HigherOrderUDFImpl>) -> HigherOrderUDF

Create a new HigherOrderUDF from a shared HigherOrderUDFImpl trait object.

pub fn inner(&self) -> &Arc<dyn HigherOrderUDFImpl>

Return the underlying HigherOrderUDFImpl trait object for this function.

pub fn with_aliases( self, aliases: impl IntoIterator<Item = &'static str>, ) -> Self

Adds additional names that can be used to invoke this function, in addition to name.

If you implement HigherOrderUDFImpl directly you should return aliases directly.

pub fn name(&self) -> &str

Returns this function’s name.

See HigherOrderUDFImpl::name for more details.

pub fn aliases(&self) -> &[String]

Returns the aliases for this function.

See HigherOrderUDF::with_aliases for more details.

pub fn schema_name(&self, args: &[Expr]) -> Result<String>

Returns this function’s schema_name.

See HigherOrderUDFImpl::schema_name for more details.

pub fn signature(&self) -> &HigherOrderSignature

Returns this function’s HigherOrderSignature.

pub fn lambda_parameters( &self, step: usize, fields: &[ValueOrLambda<FieldRef, Option<FieldRef>>], ) -> Result<LambdaParametersProgress>

Returns the parameters of all lambdas of this function for the current step.

See HigherOrderUDFImpl::lambda_parameters for more details.

pub fn coerce_values_for_lambdas( &self, fields: &[ValueOrLambda<DataType, DataType>], ) -> Result<Option<Vec<DataType>>>

Coerce value arguments based on lambda output types.

See HigherOrderUDFImpl::coerce_values_for_lambdas for more details.

pub fn return_field_from_args( &self, args: HigherOrderReturnFieldArgs<'_>, ) -> Result<FieldRef>

Returns the return field of the function given its arguments.

See HigherOrderUDFImpl::return_field_from_args for more details.

pub fn clear_null_values(&self) -> bool

Whether List or LargeList arguments should have non-empty null sublists cleaned before invoking this function.

pub fn invoke_with_args( &self, args: HigherOrderFunctionArgs, ) -> Result<ColumnarValue>

Invoke the function returning the appropriate result.

See HigherOrderUDFImpl::invoke_with_args for more details.

pub fn short_circuits(&self) -> bool

Returns true if some of this function’s subexpressions may not be evaluated.

See HigherOrderUDFImpl::short_circuits for more details.

pub fn conditional_arguments<'a>( &self, args: &'a [Expr], ) -> Option<(Vec<&'a Expr>, Vec<&'a Expr>)>

Returns which arguments are evaluated eagerly vs lazily.

See HigherOrderUDFImpl::conditional_arguments for more details.

pub fn coerce_value_types( &self, arg_types: &[DataType], ) -> Result<Vec<DataType>>

Coerce value arguments of a function call to types that the function can evaluate.

See HigherOrderUDFImpl::coerce_value_types for more details.

pub fn documentation(&self) -> Option<&Documentation>

Returns the documentation for this function, if any.