pub fn data_types(
    current_types: &[DataType],
    signature: &Signature
) -> Result<Vec<DataType>>
Returns the data types that each argument must be coerced to match signature.

See the module level documentation for more detail on coercion.