Function datafusion_expr::type_coercion::is_date
source · pub fn is_date(dt: &DataType) -> boolExpand description
Determine whether the given data type dt is a Date.
pub fn is_date(dt: &DataType) -> boolDetermine whether the given data type dt is a Date.