Function datafusion_expr::type_coercion::is_numeric
source · pub fn is_numeric(dt: &DataType) -> boolExpand description
Determine whether the given data type dt represents numeric values.
pub fn is_numeric(dt: &DataType) -> boolDetermine whether the given data type dt represents numeric values.