Function datafusion_expr::type_coercion::is_timestamp
source · pub fn is_timestamp(dt: &DataType) -> boolExpand description
Determine whether the given data type dt is a Timestamp.
pub fn is_timestamp(dt: &DataType) -> boolDetermine whether the given data type dt is a Timestamp.