Module functions

Module functions 

Traits

UDFCoercionExt
Extension trait to unify common functionality between ScalarUDF, AggregateUDF and WindowUDF for use by signature coercion functions.

Functions

can_coerce_fromDeprecated
Return true if a value of type type_from can be coerced (losslessly converted) into a value of type_to
data_typesDeprecated
Performs type coercion for function arguments.
data_types_with_scalar_udfDeprecated
Performs type coercion for scalar function arguments.
fields_with_aggregate_udfDeprecated
Performs type coercion for aggregate function arguments.
fields_with_udf
Performs type coercion for UDF arguments.
fields_with_window_udfDeprecated
Performs type coercion for window function arguments.
value_fields_with_higher_order_udf
Performs type coercion for higher order function arguments.
value_fields_with_higher_order_udf_and_lambdas
Performs type coercion for higher order function arguments, including those defined by crate::HigherOrderUDFImpl::coerce_values_for_lambdas, if it returns Some(...) instead of the default None. Note that compared to value_fields_with_higher_order_udf, this function requires the ValueOrLambda::Lambda variant to contain the output field of the lambda.