pub struct PartitionBatchState {
pub record_batch: RecordBatch,
pub is_end: bool,
pub n_out_row: usize,
}Expand description
State for each unique partition determined according to PARTITION BY column(s)
Fields
record_batch: RecordBatchThe record batch belonging to current partition
is_end: boolFlag indicating whether we have received all data for this partition
n_out_row: usizeNumber of rows emitted for this partition since the last pruning pass
Implementations
Sourceimpl PartitionBatchState
impl PartitionBatchState
pub fn new(schema: SchemaRef) -> Self
pub fn new_with_batch(batch: RecordBatch) -> Self
pub fn extend(&mut self, batch: &RecordBatch) -> Result<()>