HigherOrderTypeSignature

Enum HigherOrderTypeSignature 

pub enum HigherOrderTypeSignature {
    UserDefined,
    VariadicAny,
    Any(usize),
    Exact(Vec<ValueOrLambda<(), ()>>),
}
The types of arguments for which a function has implementations.

HigherOrderTypeSignature DOES NOT define the types that a user query could call the function with. DataFusion will automatically coerce (cast) argument types to one of the supported function signatures, if possible.

Overview

Functions typically provide implementations for a small number of different argument DataTypes, rather than all possible combinations. If a user calls a function with arguments that do not match any of the declared types, DataFusion will attempt to automatically coerce (add casts to) function arguments so they match the HigherOrderTypeSignature. See the type_coercion module for more details

Variants



UserDefined

The acceptable signature and coercions rules are special for this function.

If this signature is specified, DataFusion will call HigherOrderUDFImpl::coerce_value_types to prepare argument types.



VariadicAny

One or more lambdas or arguments with arbitrary types



Any(usize)

The specified number of lambdas or arguments with arbitrary types.



Exact(Vec<ValueOrLambda<(), ()>>)

Exactly the specified arguments in the given order, with arbitrary types. DataFusion will call HigherOrderUDFImpl::coerce_value_types to prepare the value argument types.