Structs
- Execution
Props - Holds properties and scratch state used while optimizing a
LogicalPlanand translating it into an executable physical plan, such as the statement start time used during simplification.
LogicalPlan
and translating it into an executable physical plan, such as the statement
start time used during simplification.