Module execution_props

Module execution_props 

Structs

ExecutionProps
Holds properties and scratch state used while optimizing a LogicalPlan and translating it into an executable physical plan, such as the statement start time used during simplification.
ScalarSubqueryResults
Shared results container for uncorrelated scalar subqueries.
SubqueryIndex
Index of a scalar subquery within a ScalarSubqueryResults container.