Logical Expressions: Expr
Structs
- Aggregate
Function - Aggregate function
- Aggregate
Function Params - Alias
- Alias expression
- Between
- BETWEEN expression
- Binary
Expr - Binary expression for
Expr::BinaryExpr - Case
- CASE expression
- Cast
- Cast expression
- Except
Select Item sql - Bigquery
EXCEPTinformation, with at least one column. - Exists
- EXISTS expression
- Expr
List Display - Formats a list of
&Exprwith a custom separator using SQL display format - Field
Metadata - Literal metadata
- Higher
Order Function - Invoke a
HigherOrderUDFwith a set of arguments - Ilike
Select Item sql - Snowflake
ILIKEinformation. - InList
- InList expression
- InSubquery
- IN subquery
- Lambda
- A Lambda expression with a set of parameters names and a body
- Lambda
Variable - A named reference to a lambda parameter which includes it’s own
FieldRef, which is used to implementExprSchemable, for example. It is an option only to make easier forexpr_apiusers to construct lambda variables, but any expression tree orLogicalPlancontaining unresolved variables must be resolved before usage with eitherExpr::resolve_lambda_variablesorLogicalPlan::resolve_lambda_variables. The default SQL planner produces already resolved variables and no further resolving is required. - Like
- LIKE expression
- Placeholder
- Placeholder, representing bind parameter values such as
$1or$name. - Planned
Replace Select Item - The planned expressions for
REPLACE - Replace
Select Element sql - Syntax
- Scalar
Function - Invoke a
ScalarUDFwith a set of arguments - SetComparison
- Set comparison subquery (e.g.
= ANY,> ALL) - Sort
- SORT expression
- TryCast
- TryCast Expression
- Unnest
- UNNEST expression.
- Wildcard
Options - Additional options for wildcards, e.g. Snowflake
EXCLUDE/RENAMEand BigqueryEXCEPT. - Window
Function - Window function
- Window
Function Params
Enums
- Exclude
Select Item sql - Snowflake
EXCLUDEinformation. - Expr
- Represents logical expressions such as
A + 1, orCAST(c1 AS int). - GetField
Access - Access a sub field of a nested type, such as
FieldorList - Grouping
Set - Grouping sets
- Null
Treatment - Rename
Select Item sql - Snowflake
RENAMEinformation. - SetQuantifier
- Whether the set comparison uses
ANY/SOMEorALL - Window
Function Definition - A function used as a SQL window function
Constants
Functions
- display_
comma_ separated - intersect_
metadata_ for_ union - Intersects multiple metadata instances for UNION operations.
- physical_
name - The name of the column (field) that this
Exprwill produce in the physical plan. The difference from Expr::schema_name is that top-level columns are unqualified. - schema_
name_ from_ exprs - Get schema_name for Vector of expressions
- schema_
name_ from_ sorts
Type Aliases
- Schema
Field Metadata - The metadata used in
Field::metadata.