pub struct LambdaVariable {
pub name: String,
pub field: Option<FieldRef>,
pub spans: Spans,
}Expand description
A named reference to a lambda parameter which includes it’s own FieldRef,
which is used to implement ExprSchemable, for example. It is an option only to make
easier for expr_api users to construct lambda variables, but any expression
tree or LogicalPlan containing unresolved variables must be resolved before
usage with either Expr::resolve_lambda_variables or
LogicalPlan::resolve_lambda_variables. The default SQL planner produces
already resolved variables and no further resolving is required.
After resolving, if any argument from the lambda function which this variables originates from have it’s field changed (type, nullability, metadata, etc), the resolved variable may became outdated and must be resolved again.
Fields
name: Stringfield: Option<FieldRef>spans: SpansImplementations
Sourceimpl LambdaVariable
impl LambdaVariable
Sourcepub fn new(name: String, field: Option<FieldRef>) -> Self
pub fn new(name: String, field: Option<FieldRef>) -> Self
Create a lambda variable from a name and an optional field.
If the field is none, the expression tree or LogicalPlan which
owns this variable must be resolved before usage with either
Expr::resolve_lambda_variables or LogicalPlan::resolve_lambda_variables.
pub fn spans_mut(&mut self) -> &mut Spans