pub struct RangePartitioning { /* private fields */ }Expand description
Logical range partitioning.
RangePartitioning describes an ordered logical key space with split points.
orderingdefines the partitioning key and ordering using logicalSortExprs.split_pointsdefine the boundaries between adjacent partitions.
Comparisons use the lexicographic order defined by ordering,
including ASC/DESC and null ordering. Split points must be ordered
according to that ordering, and each split point must have one value per
ordering expression. See SplitPoint for the shared boundary contract.
The expressions are resolved against the declaring plan’s schema. This constructor does not validate split point value types against the resolved expression types. Like other user-specified data properties such as sortedness, if a source declares range partitioning, it is responsible for placing each row in the partition described by the split points. DataFusion will not validate this is upheld.
NOTE: Range-aware optimizer and execution behavior will be introduced incrementally. See https://github.com/apache/datafusion/issues/22395.
Implementations
Sourceimpl RangePartitioning
impl RangePartitioning
Sourcepub fn try_new(
ordering: Vec<SortExpr>,
split_points: Vec<SplitPoint>,
) -> Result<Self>
pub fn try_new( ordering: Vec<SortExpr>, split_points: Vec<SplitPoint>, ) -> Result<Self>
Creates logical range partitioning metadata and validates split point shape and ordering.
Sourcepub fn partition_count(&self) -> usize
pub fn partition_count(&self) -> usize
Return the number of partitions.
Sourcepub fn split_points(&self) -> &[SplitPoint]
pub fn split_points(&self) -> &[SplitPoint]
Returns the ordered split points between partitions.