pub struct TableScanBuilder { /* private fields */ }Expand description
Builder for TableScan.
Prefer this over constructing a TableScan directly: it derives the
projected_schema from the source schema and projection, and is resilient
to new fields being added to TableScan. An existing scan can be turned
back into a builder with TableScanBuilder::from(scan), tweaked, and
rebuilt with TableScanBuilder::build.
Implementations
Sourceimpl TableScanBuilder
impl TableScanBuilder
Sourcepub fn new(
table_name: impl Into<TableReference>,
source: Arc<dyn TableSource>,
) -> Self
pub fn new( table_name: impl Into<TableReference>, source: Arc<dyn TableSource>, ) -> Self
Create a new builder for a scan of source named table_name.
Sourcepub fn with_projection(self, projection: Option<Vec<usize>>) -> Self
pub fn with_projection(self, projection: Option<Vec<usize>>) -> Self
Set the column projection (indices into the source schema).
Sourcepub fn with_filters(self, filters: Vec<Expr>) -> Self
pub fn with_filters(self, filters: Vec<Expr>) -> Self
Set the filter expressions offered to the table provider.
Sourcepub fn with_fetch(self, fetch: Option<usize>) -> Self
pub fn with_fetch(self, fetch: Option<usize>) -> Self
Set the maximum number of rows to read.
Sourcepub fn with_statistics_requests(
self,
statistics_requests: BTreeSet<StatisticsRequest>,
) -> Self
pub fn with_statistics_requests( self, statistics_requests: BTreeSet<StatisticsRequest>, ) -> Self
Set the statistics requests for the scan. See
TableScan::statistics_requests.