FunctionRegistry trait
Structs
- Extension
Type Registration - The registration of an extension type. Implementations of this trait are responsible for
creating instances of
DFExtensionTypethat represent the entire semantics of an extension type. - Memory
Extension Type Registry - An
ExtensionTypeRegistrythat uses in memoryHashMaps. - Memory
Function Registry - A
FunctionRegistrythat uses in memoryHashMaps
Traits
- Extension
Type Registry - Manages
ExtensionTypeRegistrations, which allow users to register custom behavior for extension types. - Function
Registry - A registry knows how to build logical expressions out of user-defined function’ names
- Serializer
Registry - Serializer and deserializer registry for extensions like UserDefinedLogicalNode.
Type Aliases
- Extension
Type Factory - A factory that creates instances of extension types from a storage
DataTypeand the metadata. - Extension
Type Registration Ref - A cheaply cloneable pointer to an ExtensionTypeRegistration.
- Extension
Type Registry Ref - A cheaply cloneable pointer to an ExtensionTypeRegistry.