pub struct MemoryExtensionTypeRegistry { /* private fields */ }Expand description
An ExtensionTypeRegistry that uses in memory HashMaps.
Implementations
Sourceimpl MemoryExtensionTypeRegistry
impl MemoryExtensionTypeRegistry
Sourcepub fn new_empty() -> Self
pub fn new_empty() -> Self
Creates an empty MemoryExtensionTypeRegistry.
Sourcepub fn new_with_canonical_extension_types() -> Self
pub fn new_with_canonical_extension_types() -> Self
Pre-registers the canonical extension types in the extension type registry.
Sourcepub fn new_with_types(
types: impl IntoIterator<Item = ExtensionTypeRegistrationRef>,
) -> Result<Self>
pub fn new_with_types( types: impl IntoIterator<Item = ExtensionTypeRegistrationRef>, ) -> Result<Self>
Creates a new MemoryExtensionTypeRegistry with the provided types.
Errors
Returns an error if one of the types is a native type.
Sourcepub fn all_extension_types(&self) -> Vec<ExtensionTypeRegistrationRef> ⓘ
pub fn all_extension_types(&self) -> Vec<ExtensionTypeRegistrationRef> ⓘ
Returns a list of all registered types.