pub struct SimplifyContext { /* private fields */ }Expand description
Provides simplification information based on schema, query execution time, and configuration options.
Example
See the simplify_demo in the expr_api example
Implementations
Sourceimpl SimplifyContext
impl SimplifyContext
Sourcepub fn with_config_options(self, config_options: Arc<ConfigOptions>) -> Self
pub fn with_config_options(self, config_options: Arc<ConfigOptions>) -> Self
Set the ConfigOptions for this context
Sourcepub fn with_schema(self, schema: DFSchemaRef) -> Self
pub fn with_schema(self, schema: DFSchemaRef) -> Self
Set the schema for this context
Sourcepub fn with_query_execution_start_time(
self,
query_execution_start_time: Option<DateTime<Utc>>,
) -> Self
pub fn with_query_execution_start_time( self, query_execution_start_time: Option<DateTime<Utc>>, ) -> Self
Set the query execution start time
Sourcepub fn with_current_time(self) -> Self
pub fn with_current_time(self) -> Self
Set the query execution start to the current time
Sourcepub fn schema(&self) -> &DFSchemaRef
pub fn schema(&self) -> &DFSchemaRef
Returns the schema
Sourcepub fn is_boolean_type(&self, expr: &Expr) -> Result<bool>
pub fn is_boolean_type(&self, expr: &Expr) -> Result<bool>
Returns true if this Expr has boolean type
Sourcepub fn get_data_type(&self, expr: &Expr) -> Result<DataType>
pub fn get_data_type(&self, expr: &Expr) -> Result<DataType>
Returns data type of this expr needed for determining optimized int type of a value
Sourcepub fn query_execution_start_time(&self) -> Option<DateTime<Utc>>
pub fn query_execution_start_time(&self) -> Option<DateTime<Utc>>
Returns the time at which the query execution started.
If None, time-dependent functions like now() will not be simplified.
Sourcepub fn config_options(&self) -> &Arc<ConfigOptions>
pub fn config_options(&self) -> &Arc<ConfigOptions>
Returns the configuration options for the session.