HigherOrderFunctionArgs

Struct HigherOrderFunctionArgs 

pub struct HigherOrderFunctionArgs {
    pub args: Vec<ValueOrLambda<ColumnarValue, LambdaArgument>>,
    pub arg_fields: Vec<ValueOrLambda<FieldRef, FieldRef>>,
    pub number_rows: usize,
    pub return_field: FieldRef,
    pub config_options: Arc<ConfigOptions>,
}
Arguments passed to HigherOrderUDFImpl::invoke_with_args when invoking a higher order function.

Fields

args: Vec<ValueOrLambda<ColumnarValue, LambdaArgument>>

The evaluated arguments and lambdas to the function

arg_fields: Vec<ValueOrLambda<FieldRef, FieldRef>>

Field associated with each arg, if it exists For lambdas, it will be the field of the result of the lambda if evaluated with the parameters returned from HigherOrderUDFImpl::lambda_parameters

number_rows: usize

The number of rows in record batch being evaluated

return_field: FieldRef

The return field of the higher order function returned (from return_field_from_args) when creating the physical expression from the logical expression

config_options: Arc<ConfigOptions>

The config options at execution time

Implementations

Source

impl HigherOrderFunctionArgs

pub fn return_type(&self) -> &DataType

The return type of the function. See Self::return_field for more details.