Module datafusion_expr::type_coercion::binary
source · Expand description
Coercion rules for matching argument types for binary operators
Functions
- Coerce
lhs_typeandrhs_typeto a common type for the purposes of a comparison operation - Returns the coerced input types for a binary expression evaluating the
opwith the left and right hand types - returns the resulting type of a binary expression evaluating the
opwith the left and right hand types - Returns the wider type among arguments
lhsandrhs. The wider type is the type that can safely represent values from both types without information loss. Returns an Error if types are incompatible. - coercion rules for like operations. This is a union of string coercion rules and dictionary coercion rules
- coercion rules for regular expression comparison operations. This is a union of string coercion rules and dictionary coercion rules