Coercion rules for matching argument types for binary operators

Functions

  • Returns the return type of a binary operator or an error when the binary operator cannot perform the computation between the argument’s types, even after type coercion.
  • Coercion rules for all binary operators. Returns the output type of applying op to an argument of lhs_type and rhs_type.
  • Returns the output type of applying comparison operations such as eq, not eq, lt, lteq, gt, and gteq to arguments of lhs_type and rhs_type.
  • coercion rules for like operations. This is a union of string coercion rules and dictionary coercion rules
  • coercion rules for regular expression comparison operations. This is a union of string coercion rules and dictionary coercion rules
  • Return the output type from performing addition or subtraction operations on temporal data types