Functions

Return true if a value of type type_from can be coerced (losslessly converted) into a value of type_to
Performs type coercion for functions Returns the data types that each argument must be coerced to match signature.